/************************************************************/
/*                                                          */
/*  This code demonstrates:                                 */
/*  (a) How to configure the I_CACHE/D_CACHE on BF-533      */
/*  (b) Effects of enabling I_CACHE/D_CACHE on performance  */
/*                                                          */
/*  The code perform matrix addition on data_A & data_B     */
/*  The data_A and data_B resides in the external memory.   */
/*  Result Matrix is also stored in the External memory     */
/*                                                          */
/*  The subroutine to perform addition resides in the       */
/*  external memory                                         */
/*                                                          */
/*  The External Memory Can be configured as cacheable for  */
/*  Instruction and Data by defining the labels             */
/*  ENABLE_I_CACHE and ENABLE_D_CACHE in the "main.h"       */
/*                                                          */
/*                              Initial Version             */
/*                              Kunal Singh                 */
/*                              DSP applications, ADI       */
/*                              Dated 3 April, 2005         */
/*                                                          */
/*                              Re-Release                  */
/*                              for VisualDSP++ 5.0         */
/*                              Andreas Pellkofer           */
/*                              GPDSP Applications, ADI     */
/*                              Dated 9 March, 2009         */
/*                                                          */
/************************************************************/

#include <blackfin.h>
#include <services_types.h>
#include <stdio.h>
#include "main.h"

#define DCPLB_CNT 10
#define ICPLB_CNT 9
 
/************************************************/
/*                                              */
/*      Entries for the DCPLBx_ADDR             */
/*                                              */
/************************************************/
#pragma section ("L1_data_b")

u32 D_cplb_addrs[DCPLB_CNT] =
    {
        0xff800000,
        0xff900000, 
        0x00000000,
        0x00400000,
        0x00800000, 
        0x00C00000, 
        0x01000000,
        0x01400000,
        0x01800000, 
        0x01C00000 
    };

/************************************************/
/*                                              */
/*      Entries for the DCPLBx_DATA             */
/*                                              */
/************************************************/
#pragma section ("L1_data_b")

u32 D_cplb_data[DCPLB_CNT] =
    {
        0x2001f,
        0x2001f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f,
        0x3D01f
    };

/************************************************/
/*                                              */
/*      Entries for the ICPLBx_ADDR             */
/*                                              */
/************************************************/
#pragma section ("L1_data_b")

u32 I_cplb_addrs[9] =
    {
        0xFFA00000, 
        0x00000000, 
        0x00400000, 
        0x00800000, 
        0x00C00000, 
        0x01000000, 
        0x01400000, 
        0x01800000, 
        0x01C00000
    };

/************************************************/
/*                                              */
/*      Entries for the ICPLBx_DATA             */
/*                                              */
/************************************************/
#pragma section ("L1_data_b")

u32 I_cplb_data[9] =
    {
        0x00020003,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005,
        0x00031005
    };

/********************************/
/*                              */
/*  Lock Control for the ICache */
/*      1: Lock the way         */
/*      0: Don't Lock           */
/*                              */
/********************************/
#pragma section ("L1_data_b")

u32 Lock_Control[4] =
    {
        0,
        0,
        0,
        0
    };


#pragma section("sdram0")
u32 data_A [ARRAY_LENGTH];


#pragma section("sdram2")
u32 data_B [ARRAY_LENGTH];


#pragma section("sdram2")
u32 Result [ARRAY_LENGTH];


volatile u32 decLow, decHigh;

u32 cache;

#pragma section("L1_code")

void main()
{
    u32 i;
    u32 dcplb_cnt, icplb_cnt, d_bank_cnt, disable_lru;

    /****************************************************/
    /* Configure the DCPLBs and Enabled the Data Cache  */
    /****************************************************/
#ifdef ENABLE_D_CACHE
    dcplb_cnt = DCPLB_CNT;
    #ifdef BOTH_D_CACHE
        d_bank_cnt = 2;
    #else
        d_bank_cnt = 1;
    #endif
    Config_D_Cache(D_cplb_addrs, D_cplb_data, dcplb_cnt, d_bank_cnt);
#endif


    /****************************************************/
    /* Configure the DCPLBs and Enabled the Data Cache  */
    /* Enabled all ways                                 */
    /* Use Modified LRU policy                          */
    /****************************************************/
#ifdef ENABLE_I_CACHE
    icplb_cnt = ICPLB_CNT;
    disable_lru = 0;
    Config_I_Cache(I_cplb_addrs, I_cplb_data, icplb_cnt, disable_lru, Lock_Control);
#endif

    /******************************/
    /* Initialize the data arrays */
    /******************************/
    for (i = 0; i<ARRAY_LENGTH; i++)
    {
        data_A[i] = i;
        data_B[i] = i;
    }

    /********************************************/
    /* Clear the CYCLES and CYCLES2 register    */
    /********************************************/
    StartProfiling ();

    matrix_add(Result, data_A, data_B, ARRAY_LENGTH);

    /********************************************/
    /* Read back the CYCLES & CYCLES2 register  */
    /********************************************/
    EndProfiling (&decLow, &decHigh);

    /****************************/
    /*  Print the Cycle Count   */
    /****************************/
    printf("\n The cycle count is  %u: ",decLow);
}
